#ifndef LLIST_H
#define LLIST_H
#include <stddef.h>
#include <assert.h>

typedef struct llist_t llist_t;
struct llist_t{
    llist_t * left;
    llist_t * right;
};

static inline int
llist_empty(const llist_t * list)
{
    return list->left == list;
}

static inline void
llist_init(llist_t *list)
{
    list->right = list->left = list;
}

static inline void 
llist_add(llist_t * left, llist_t * right)
{
    llist_t * old;

    // first save the pointer from the left->rigth
    old = left->right;

    // now left->right shall point to right
    left->right = right;
    
    // now the old left pointer shall point to what the
    // new right->left was pointing
    old->left = right->left;

    // now right->left shall contain the left
    right->left = left;

    // finally 
    old->left->right = old;
}

static inline void
llist_drop(llist_t * el)
{
    if(llist_empty(el)) return;
    el->left->right = el->right;
    el->right->left = el->left;
    llist_init(el);
}

// this is pointer to the left element in chain
static inline llist_t *
llist_left(const llist_t * list)
{
    return list->left;
}

// this one is a pointer to the right element in chain
static inline llist_t *
llist_right(const llist_t *list)
{
    return list->right;
}


#endif
