#!/usr/bin/perl

# this script can be used to check where the timer would be placed into
# the grid, given some state of the grid.

use strict;
use integer;

use Getopt::Long;

use constant TMR_GROUP_SIZE => 64;
use constant TMR_GROUPS_CNT => 64;
use constant TMR_LOOP => TMR_GROUPS_CNT * TMR_GROUP_SIZE;

my $sid = 0;
my $gid = 0;
my $delay = 0;
my $p = 0;

die "\n" unless GetOptions("sid=i" => \$sid, "gid=i" => \$gid,
                   "delay=i" => \$delay, "p=i" => \$p);

my @sec;
my @grp;

if($p < TMR_GROUP_SIZE)
{
    my $slot = ($sid + $p) % TMR_GROUP_SIZE;
    $delay = $p if($p < $delay);
    print "Goes in $slot:$gid. _period = 0. New delay is $delay\n";
    $sec[$slot] = 'val';
}
else
{
    my $np = $p - TMR_GROUP_SIZE + $sid;
    my $l = $np / TMR_LOOP;
    $np %= TMR_LOOP;
    my $g = $np / TMR_GROUP_SIZE;
    my $s = $np % TMR_GROUP_SIZE;
    my $slot = ($gid + $g) % TMR_GROUP_SIZE;
    my $period = $l * TMR_LOOP + $s;
    my $ndelay = ($g + 1) * TMR_GROUP_SIZE - $sid;
    $delay = $ndelay if $ndelay < $delay;
    print "Goes in group slot $slot. _period = $period. New delay is $delay\n";
    $grp[$slot] = $period;
}

my $elapsed = 0;
while($sid < 64)
{
    if(defined $sec[$sid])
    {
        print "Elapsed: $elapsed\n";
        last;
    }
    $sid++;
    $elapsed++;
    if($sid == 64)
    {
        $sid = 0;
        if(defined $grp[$gid])
        {
            if($grp[$gid] > TMR_LOOP)
            {
                $grp[$gid] -= TMR_LOOP;
            }
            else
            {
                $sec[$grp[$gid]] = 'val';
                $grp[$gid] = undef;
            }
        }
        $gid++;
        $gid = 0 if $gid == 64;
    }
}

