#include "evtid.h"
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <stdlib.h>
#include <error.h>
#include <errno.h>
#include <fcntl.h>
#include <stdio.h>
#include <unistd.h>

static int socks[2];
static evti_t tmr;
static evd_t ev_0;

// first reader shall get blocked read and request read checking.
// the first writer shall be called immidiately and fill the buffer till
// writing is not possible any more. After that it shall request writing
// and then push some data to the other side. The second reader shall
// wake up and read part of the data of the peer. After that it shall
// exit and second writer shall be called.
//

static void
reader_3(evd_t * evd)
{
    error(EXIT_FAILURE, 0, "Didn't expect this handler to be called");
}

static void
do_reading(evd_t * evd)
{
    ssize_t chk;
    char buf[1024];
    chk = read(socks[0], buf, sizeof(buf));
    if(chk >= 0) return;

    if(errno != EAGAIN && errno != EWOULDBLOCK)
        error(EXIT_FAILURE, errno, "Unexpected error at this stage");
    fputs("Finished emptying buffer\n", stdout);
    evd_nowatch(evd);
}

static void
empty_sock(evti_t * tmr)
{
    evd_init(&ev_0);
    ev_0.d = socks[0];
    ev_0.raction = do_reading;
    evd_watch(&ev_0);
    evd_raction_active(&ev_0);
}

static void
reader_2(evd_t * evd)
{
    fputs("Reader 2 is called. Disabling readinnes report and getting some data from other end\n", stdout);

    evd_raction_inactive(evd);
    evd->raction = reader_3;

    evti_init(&tmr);
    tmr.action = empty_sock;
    evti_start_s(&tmr, 2);
}

static void
reader_1(evd_t * evd)
{
    ssize_t chk;
    char buf[1024];

    fputs("Reader 1 is called. Setting reader 2 as handler\n", stdout);

    evd->raction = reader_2;

    chk = read(evd->d, buf, sizeof(buf));
    if(chk < 0)
    {
        if(errno == EAGAIN || errno == EWOULDBLOCK)
        {
            fputs("Sock read would block\n", stdout);
            evd_input_exhausted(evd);
            return;
        }
    }
    error(EXIT_FAILURE, errno, "Expected the blocking read at this stage");
}

static void
writer_2(evd_t * evd)
{
    fputs("Writer 2 is called. SUCCESS\n", stdout);
    evd_nowatch(evd);
}

static void
writer_1(evd_t * evd)
{
    ssize_t chk;
    char buf[1024] = {0};

    chk = write(evd->d, buf, sizeof(buf));
    if(chk >= 0) return evd_call_waction(evd);

    if(errno != EAGAIN && errno != EWOULDBLOCK)
        error(EXIT_FAILURE, errno, "Didn't get expected error");

    fputs("Writer 1 is done filling up the buffer\n", stdout);

    evd_check_waction(evd);
    // now inject some junk to the other side
    chk = write(socks[0], buf, 150);
    if(chk < 0) error(EXIT_FAILURE, errno, "Can't write to the other side");

    evd->waction = writer_2;
    fputs("Writer 1 is done\n", stdout);
}

int
main(void)
{
    int chk;
    evd_t evd;

    chk = socketpair(AF_UNIX, SOCK_STREAM, 0, socks);
    if(chk != 0) error(EXIT_FAILURE, errno, "Can't creates sockets pair");

    if(fcntl(socks[0], F_SETFL, O_NONBLOCK) != 0)
        error(EXIT_FAILURE, errno, "Failed to set non-blocking operation for descriptor");
    if(fcntl(socks[1], F_SETFL, O_NONBLOCK) != 0)
        error(EXIT_FAILURE, errno, "Failed to set non-blocking operation for descriptor");

    evtid_init();

    evd_init(&evd);
    evd.d = socks[1];
    evd.raction = reader_1;
    evd.waction = writer_1;
    evd_watch(&evd);
    evd_raction_active(&evd);
    evd_check_waction(&evd);

    evtid_dispatcher();

    evd_fini(&evd);

    evtid_fini();
    return 0;
}
